/*
 * BlockingPing.java 
 *
 * Copyright (c) Verax Systems.
 * All rights reserved.
 *
 * This software is furnished under a license. Use, duplication,
 * disclosure and all other uses are restricted to the rights
 * specified in the written license agreement.
 */
package com.veraxsystems.icmp.ping.handlers;

import com.veraxsystems.icmp.IcmpStatus;
import com.veraxsystems.icmp.jni.IcmpEchoResponse;
import com.veraxsystems.icmp.ping.Ping;
import com.veraxsystems.icmp.ping.PingResponseHandler;
import com.veraxsystems.icmp.ping.PingResult;
import com.veraxsystems.icmp.ping.exception.PingErrorException;
import com.veraxsystems.icmp.util.TimeUtil;
import java.net.InetAddress;
import java.util.Map;
import java.util.TreeMap;
import java.util.concurrent.CountDownLatch;
import java.util.logging.Level;
import java.util.logging.Logger;


/**
 * The implementation of {@link PingResponseHandler} for synchronous processing of ICMP EchoRequests
 * using asynchronous {@link Ping#echo Ping.echo} method. 
 * The synchronous (blocking) {@link BlockingPing#echo BlockingPing.echo} method is defined by this class.
 * It sends several ICMP EchoRequests to the set of address provided as input parameter. At the end of
 * request processing the appropriate overridden methods of <code>PingResponseHandler</code> are executed. 
 * Each received result is converted to {@link PingResult} object and assigned to appropriate 
 * entry of internal result array. Additionally, the count down semaphore is decreased
 * so <code>BlockingPing.echo</code> method waits until all requests are processed - this way the 
 * synchronous behavior is achieved.
 */
public class BlockingPing implements PingResponseHandler
{
   private PingResult[] result = null;
   private Map<Integer, Integer> requestsMap = null;
   private CountDownLatch countDown = null;

   
   /*
    * Assign result to corresponding array entry. The array index is retrieved by <code>messageId</code>
    * from index map created when rewuests were sent.
    * The CountDownLatch counter is decreased so on last result the main thread (blocked
    * echo method) may contimue.
    */
   private void setResult(int messageId, PingResult pingResult)
   {
      synchronized(requestsMap)
      {
          Integer resultIndex = requestsMap.get(messageId);
          // In general it is impossible to have "null" here
          if (resultIndex != null)
              result[resultIndex.intValue()] = pingResult;
          else 
              Logger.getLogger(Ping.class.getName()).log(Level.SEVERE, "Unexpected bahavior");
          countDown.countDown();
      };
      return;
   }
   
   /*
    * Handle reponse received. Result is put into result array using <code>setResult<method>
    */
   public void handleResponse(PingRequest request, IcmpEchoResponse response) 
   {
      setResult(request.getMessageId(), new PingResult(request.getAddress(), response.getStatus(), PingResult.ResultType.Response, request.getSendTime(), response.getResponseTime()));
      return;
   }
   
   /*
    * Handle reponse timeout. Result is put into result array using <code>setResult<method>
    */
   public void handleTimeout(PingRequest request) 
   {
      setResult(request.getMessageId(), new PingResult(request.getAddress(), IcmpStatus.UnexpectedResponse, PingResult.ResultType.Timeout, request.getSendTime(), TimeUtil.currentTimeMillis()));
      return;
   }
   
   /*
    * Handle reponse error. Result is put into result array using <code>setResult<method>
    */
   public void handleException(PingRequest request, Exception exception) 
   {
      setResult(request.getMessageId(), new PingResult(request.getAddress(), IcmpStatus.UnexpectedResponse, PingResult.ResultType.Error, request.getSendTime(), TimeUtil.currentTimeMillis()));
      return;
   }
   
   /**
    * Send the set of ICMP echo requests. Method is executed synchronously and waits until all echo requests
    * are finished. Internally, echo requests are send asynchronously - it means all requests are send immediately
    * without waiting for responses (those, are handled in separate thread). Next, method waits until either all echo 
    * responses are received or timeouts occurs.
    * <p>
    * Each echo result is placed in array of results corresponding to <code>addresses</code> parameter (it means for each input host 
    * the result is placed in corresponding element of result array). 
    * <p>
    * Each request timeout is calculated separately - it means that its calculation starts in the moment
    * when request is sent (not when <code>echo</code> method is called). Such behavior leads to 
    * situation that <code>echo</code> execution lasts slightly longer than timeout specified.
    * <p>
    * Method reports exception when either {@link Ping} is not initialized or current thread is interrupted.
    * @param addresses array of host addresses send ICMP echo requests
    * @param dataToSend data to be send as part of ICMP message (when null the default data is used)
    * @param dataLength length of data to be sent 
    * @param timeout expressed in milliseconds time to wait for echo response
    * @return the array of results
    * @throws PingErrorException issued if Ping class is not initialized or thread is interrupted  
    */
   public PingResult[] echo(InetAddress[] addresses, byte[] dataToSend, int dataLength, int timeout) throws PingErrorException 
   {
      if (addresses != null)
      {
         try
         {
            // Initialize waiting counter so program can wait until all requests are handled
            countDown = new CountDownLatch(addresses.length);
            result = new PingResult[addresses.length];
            requestsMap = new TreeMap<Integer, Integer>();
            for(int i = 0; i < addresses.length; i++)
            {
               int messageId;
               
               synchronized(requestsMap)
               {
                   if (dataToSend != null)
                       messageId = Ping.echo(addresses[i], dataToSend, timeout, this);
                   else
                       messageId = Ping.echo(addresses[i], dataLength, timeout, this);
                   requestsMap.put(messageId, i);
               }
            }
         }
         catch (PingErrorException ex) 
         {
            throw new PingErrorException("Echo request failure", ex);
         };
         try 
         {
            countDown.await();
         }
         catch (InterruptedException ex) 
         {
            throw new PingErrorException("Echo interrupted", ex);
         };
      }
      else  throw new PingErrorException("Illegal parameter", null);
      return result;
   }
}
