/*
 * Sender.java 
 *
 * Copyright (c) Verax Systems.
 * All rights reserved.
 *
 * This software is furnished under a license. Use, duplication,
 * disclosure and all other uses are restricted to the rights
 * specified in the written license agreement.
 */
package com.veraxsystems.icmp.ping.handlers;

import com.veraxsystems.icmp.jni.IcmpJniBridge;
import com.veraxsystems.icmp.jni.exception.IcmpSystemException;
import com.veraxsystems.icmp.ping.Ping;
import com.veraxsystems.icmp.ping.exception.PingErrorException;

import java.util.Queue;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;


/**
 * An ICMP sending thread implementation. 
 * The class is a {link IcmpJniBridge} sender used in {@link Ping} class.
 * It handles list of ICMP requests that are registered by {@link Sender#registerRequest registerRequest} method.
 * Once request is registered by, it is sent in thread activity loop and than passed to {@link Receiver} for future
 * handling.
 */
public class Sender implements Runnable 
{
   private IcmpJniBridge icmp = null;
   private Queue<PingRequest> requestsQueue = null;
   private Lock mutex = null;
   private Condition hasData = null;
   private Receiver receiver;

   /**
    * Constructor accepts <code>IcmpJniBridge</code> object that is used to send ICMP messages and
    * <code>Receiver</code> that is used to pass request for further handling.
    * Because ICMP echo is send request is forwarded to 
    * @param receiverVal receiving ICMP response handler
    * @param icmpVal ICMP socket
    */
   public Sender(Receiver receiverVal, IcmpJniBridge icmpVal) 
   {
      icmp = icmpVal;
      receiver = receiverVal;
      requestsQueue = new ConcurrentLinkedQueue<PingRequest>();
      mutex = new ReentrantLock();
      hasData = mutex.newCondition();
      return;
   }

   /**
    * Register ICMP EchoRequest and put it into requests queue. 
    * @param pingRequest the ICMP descriptor
    * @throws PingErrorException exception is thrown if <code>Sender</code> object is disposed
    */
   public void registerRequest(PingRequest pingRequest) throws PingErrorException 
   {
      mutex.lock();
      try 
      {
         if (requestsQueue != null) 
         {
            requestsQueue.add(pingRequest);
            hasData.signal();
         }
         else throw new PingErrorException("Not initialized:", null);
      }
      finally
      {
         mutex.unlock();
      };
      return;
   }

   /*
    * Main activity of <code>Sender</code> thread.
    */
   public void run() 
   {
      PingRequest request = null;
      
      try 
      {
         for (;;) 
         {
            // Peek request (wait for request if queue is empty). 
            // Request remains in queue until it is successfully registered in receiver.
            request = peekRequest();
            // Register request in receiver so receiver expects its response.
            // If operation succeeds, request is temporary stored in two queues.
            request.updateSendTime();
            receiver.registerRequest(request);
            // Thread.sleep(10);
            // After request is registered in receiver, it ma be removed from sender queue.
            // The code between "receiver.registerRequest" and "requestsQueue.poll" is not
            // interruptable, so we can be sure the request is not processed by two times 
            requestsQueue.poll();
            // Send echo. 
            icmp.send(request.getAddress(), request.getMessageId(), request.getData(), request.getDataLength());
         }
      } 
      catch (PingErrorException ex) 
      {
         cleanupRequestsQueue(ex);
      }
      catch (IcmpSystemException ex) 
      {
         cleanupRequestsQueue(ex);
      }
      catch (InterruptedException ex) 
      {
         cleanupRequestsQueue(ex);
      }
      finally 
      {
         dispose();
      }
   }

   /*
    * In case of socket error, whole processing is terminated and request queue is emptied.
    * All no handler requests have executed exception handler.
    */
   private void cleanupRequestsQueue(Exception ex) 
   {
      PingRequest request;
      
      mutex.lock();
      try 
      {
         request = requestsQueue.poll();
         while (request != null) 
         {
            request.getHandler().handleException(request, new PingErrorException("Operation terminated", ex));
            request = requestsQueue.poll();
         };
         requestsQueue = null;
      } 
      finally 
      {
         mutex.unlock();
      }
      return;
   }


   /*
    * Simple resources releasing
    */
   private void dispose() 
   {
      icmp = null;
      receiver = null;
      requestsQueue = null;
      hasData = null;
      mutex = null;
      return;
   }


   /*
    * Get reference of first waiting request (it is not removed from queue).  
    * If there is no reuest in queue, operation waits until it is available.
    */
   private PingRequest peekRequest() throws InterruptedException 
   {
      PingRequest result;
      
      mutex.lock();
      try 
      {
         while (requestsQueue.size() == 0) 
         {
            hasData.await();
         };
         result = requestsQueue.peek();
      } 
      finally
      {
         mutex.unlock();
      };
      return result;
   }
   
}
