/*
 * IcmpLibTest.c
 *
 * Copyright (c) Verax Systems.
 * All rights reserved.
 *
 * This software is furnished under a license. Use, duplication,
 * disclosure and all other uses are restricted to the rights
 * specified in the written license agreement.
 */
#include <windows.h>
#include <stdio.h>
#include "Icmp.h"
#include "IcmpMutex.h"

#define SUCCESS_IP     "127.0.0.1"
#define NO_ANSWER_IP   "2.3.4.5"
#define UNRECHABLE_IP  "10.0.0.1"
#define ICMP_ID        123
#define ICMP_SEQ      987
#define ICMP_DATA      "ICMP data for test"
#define ICMP_DATA_LEN  strlen(ICMP_DATA)

static int testSucceddedCount = 0;
static int testFailedCount = 0;



/*
 * Initialize tests counters
 */
static void testsInit(void)
{
   testSucceddedCount = 0;
   testFailedCount = 0;
   return;
} /* testsInit */


/*
 * Finish tests and print statistics.
 */
static void testsExit(void)
{
   printf("----------------------------------------------------\n");
   printf("Tests summary: run %d, succeded = %d, failed %d\n", testSucceddedCount + testFailedCount, testSucceddedCount, testFailedCount);
   return;
} /* testsExit */


/*
 * Print test name
 */
static void testInfo(const char *testName)
{
   printf("TEST: %s\n", testName);
   return;
} /* testInfo */


/*
 * Display information message followed by test (subtest) result.
 * Update test counters.
 */
static void checkResult(const char *testName, int result)
{
   if (result)
   {
      printf("    %s  - OK \n", testName);
      testSucceddedCount += 1;
   }
   else
   {
      printf("    %s  - failed \n", testName); 
      testFailedCount += 1;
   };
   return;
} /* checkResult */


/*
 * Mutex create/close simple test
 */
static void testMutex_simple(void)
{
   HANDLE handle;

   testInfo("testMutex_simple");
   handle = IcmpCreateMutex();
   checkResult("create mutex", handle != NULL);
   checkResult("close mutex", IcmpCloseMutex(handle));
   return;
} /* testMutex_simple */


/*
 * Mutex create/close double test
 */
static void testMutex_double(void)
{
   HANDLE handle1, handle2;

   testInfo("testMutex_double");
   handle1 = IcmpCreateMutex();
   checkResult("create mutex1 ", handle1 != NULL);
   handle2 = IcmpCreateMutex();
   checkResult("create mutex2", handle2 != NULL);
   checkResult("mutexes differ", handle1 != handle2);
   checkResult("close mutex2", IcmpCloseMutex(handle2));
   checkResult("close mutex1", IcmpCloseMutex(handle1));
   return;
} /* testMutex_double */


/*
 * Basic test for ICMP socket creation and closing.
 */
static void testCreateCloseSocket_success(void)
{
   SOCKET socket = IcmpCreateSocket();
   
   testInfo("testCreateCloseSocket_success");
   checkResult("create socket", socket != SOCKET_ERROR);
   checkResult("close socket", IcmpCloseSocket(socket) != SOCKET_ERROR);
   return;
} /* testCreateCloseSocket_success */


/*
 * Test IcmpGetSocketError for wrong input parameter (socket)
 */
static void testGetSocketError_wrongSocket(void)
{
   int result = IcmpGetSocketError((SOCKET)-1);
   
   testInfo("testGetSocketError_wrongSocket");
   checkResult("error test", result == WSAEINVAL);
   return;
} /* testGetSocketError_wrongSocket */


/*
 * Test IcmpGetErrorString 
 */
static void testGetErrrorString(void)
{
   LPTSTR errorStr;

   IcmpCloseSocket((SOCKET)-1);
   errorStr = IcmpGetErrorString(IcmpGetLastError());
   testInfo("testGetSocketError_wrongSocket");
   printf(" PRINT: %s\n", errorStr);
   IcmpReleaseErrorString(errorStr);
   return;
} /* testGetErrrorString */


/*
 * Test IcmpGetSocketError for proper docket handle
 */
static void testGetSocketError_success(void)
{
   SOCKET socket = IcmpCreateSocket();
   int result = IcmpGetSocketError(socket);
   
   testInfo("testGetSocketError_success");
   checkResult("error test", result == 0);
   IcmpCloseSocket(socket);
   return;
} /* testGetSocketError_success */


/*
 * Test IcmpSendEcho function providing proper set of parameters
 */
static void testSendEcho_success(void)
{
   SOCKET socket = IcmpCreateSocket();
   struct sockaddr_in dstAddr;
  
   testInfo("testSendEcho_success");
   checkResult("create socket", socket != SOCKET_ERROR);
   memset(&dstAddr, 0, sizeof(dstAddr));
   dstAddr.sin_addr.s_addr = inet_addr(SUCCESS_IP);
   checkResult("send echo", IcmpSendEcho(socket, (INT8 *)&dstAddr.sin_addr.S_un.S_addr, ICMP_ID, ICMP_SEQ, ICMP_DATA, ICMP_DATA_LEN)  != SOCKET_ERROR);
   checkResult("close socket", IcmpCloseSocket(socket) != SOCKET_ERROR);
   return;
} /* testSendEcho_success */


/*
 * Test IcmpSendEcho function providing illegal socket handle
 */
static void testSendEcho_wrongSocket(void)
{
   SOCKET socket = (SOCKET)-1;
   struct sockaddr_in dstAddr;
  
   testInfo("testSendEcho_wrongSocket");
   memset(&dstAddr, 0, sizeof(dstAddr));
   dstAddr.sin_addr.s_addr = inet_addr(SUCCESS_IP);
   checkResult("send echo", IcmpSendEcho(socket, (INT8 *)&dstAddr.sin_addr.S_un.S_addr, ICMP_ID, ICMP_SEQ, ICMP_DATA, ICMP_DATA_LEN)  == SOCKET_ERROR);
   checkResult("check result", IcmpGetLastError() == WSAENOTSOCK);
   return;
} /* testSendEcho_wrongSocket */


/*
 * Test IcmpSendEcho function providing wrong data buffer
 */
static void testSendEcho_illegalData(void)
{
   SOCKET socket = IcmpCreateSocket();
   struct sockaddr_in dstAddr;
  
   testInfo("testSendEcho_illegalData");
   memset(&dstAddr, 0, sizeof(dstAddr));
   dstAddr.sin_addr.s_addr = inet_addr(SUCCESS_IP);
   checkResult("send echo", IcmpSendEcho(socket, (INT8 *)&dstAddr.sin_addr.S_un.S_addr, ICMP_ID, ICMP_SEQ, (char *)-1, 4)  == SOCKET_ERROR);
   checkResult("check result", IcmpGetLastError() == WSAEINVAL);
   checkResult("close socket", IcmpCloseSocket(socket) != SOCKET_ERROR);
   return;
} /* testSendEcho_illegalData */


/*
 * Test IcmpSendEcho function providing wrong address buffer
 */
static void testSendEcho_illegalAddress(void)
{
   long illegalAddr = -1;
   SOCKET socket = IcmpCreateSocket();
  
   testInfo("testSendEcho_illegalAddress");
   checkResult("send echo", IcmpSendEcho(socket, (INT8 *)&illegalAddr, ICMP_ID, ICMP_SEQ, ICMP_DATA, ICMP_DATA_LEN)  == SOCKET_ERROR);
   checkResult("check result", IcmpGetLastError() == WSAEACCES);
   checkResult("close socket", IcmpCloseSocket(socket) != SOCKET_ERROR);
   return;
} /* testSendEcho_illegalAddress */


/*
 * Test IcmpSendEcho function providing wrong address buffer
 */
static void testSendEcho_noAnswer(void)
{
   SOCKET socket = IcmpCreateSocket();
   struct sockaddr_in dstAddr;
  
   testInfo("testSendEcho_noAnswer");
   checkResult("create socket", socket != SOCKET_ERROR);
   memset(&dstAddr, 0, sizeof(dstAddr));
   dstAddr.sin_addr.s_addr = inet_addr(NO_ANSWER_IP);
   checkResult("send echo", IcmpSendEcho(socket, (INT8 *)&dstAddr.sin_addr.S_un.S_addr, ICMP_ID, ICMP_SEQ, ICMP_DATA, ICMP_DATA_LEN)  != SOCKET_ERROR);
   checkResult("wait",  IcmpWait(socket, 500) == 0);
   checkResult("close socket", IcmpCloseSocket(socket) != SOCKET_ERROR);
   return;
} /* testSendEcho_illegalAddress */


/*
 * Test IcmpWait function providing proper set of parameters. The echo request
 * is sent to local host, so response is expected to be received.
 */
static void testWaitForResponse_success(void)
{
   SOCKET socket = IcmpCreateSocket();
   struct sockaddr_in dstAddr;
  
   testInfo("testWaitForResponse_success");
   checkResult("create socket", socket != SOCKET_ERROR);
   memset(&dstAddr, 0, sizeof(dstAddr));
   dstAddr.sin_addr.s_addr = inet_addr(SUCCESS_IP);
   checkResult("send echo", IcmpSendEcho(socket, (INT8 *)&dstAddr.sin_addr.S_un.S_addr, ICMP_ID, ICMP_SEQ, ICMP_DATA, ICMP_DATA_LEN)  != SOCKET_ERROR);
   checkResult("wait",  IcmpWait(socket, 5000) == 1);
   checkResult("close socket", IcmpCloseSocket(socket) != SOCKET_ERROR);
   return;
} /* testWaitForResponse_success */


/*
 * Test IcmpWait function providing proper set of parameters.
 * No input message ie expected so timeout occurs.
 */
static void testWaitForResponse_timeout(void)
{
   SOCKET socket = IcmpCreateSocket();
      
   testInfo("testWaitForResponse_timeout");
   checkResult("create socket", socket != SOCKET_ERROR);
   checkResult("wait",  IcmpWait(socket, 5000) == 0);
   checkResult("close socket", IcmpCloseSocket(socket) != SOCKET_ERROR);
   return;
} /* testWaitForResponse_timeout */


/*
 * Test IcmpWait function providing illegal socket handle.
 */
static void testWaitForResponse_illegalSocket(void)
{
   testInfo("testWaitForResponse_illegalSocket");
   checkResult("wait",  IcmpWait((SOCKET)-1, 5000) == SOCKET_ERROR);
   return;
} /* testWaitForResponse_illegalSocket */


/*
 * Test IcmpResponseMessage function providing proper set of parameters.
 * The echo request is sent so after a while response is received.
 */
static void testReadMessage_success(void)
{
   SOCKET socket = IcmpCreateSocket(); 
   IcmpResponseMessage response;
   struct sockaddr_in dstAddr;
  
   testInfo("testReadMessage_success");
   checkResult("create socket", socket != SOCKET_ERROR);
   memset(&dstAddr, 0, sizeof(dstAddr));
   dstAddr.sin_addr.s_addr = inet_addr(SUCCESS_IP);
   checkResult("send echo", IcmpSendEcho(socket, (INT8 *)&dstAddr.sin_addr.S_un.S_addr, ICMP_ID, ICMP_SEQ, ICMP_DATA, ICMP_DATA_LEN)  != SOCKET_ERROR);
   checkResult("wait",  IcmpWait(socket, 5000) == 1);
   checkResult("read", IcmpReadMessage(socket, &response) != SOCKET_ERROR);
   checkResult("close socket", IcmpCloseSocket(socket) != SOCKET_ERROR);
   return;
} /* testReadMessage_success */


/*
 * Test IcmpResponseMessage function providing illegal response buffer.
 * The echo request is sent so after a while response is received causing
 * function to fail on response buffer access.
 */
static void testReadMessage_illegalBuffer(void)
{
   SOCKET socket = IcmpCreateSocket(); 
   struct sockaddr_in dstAddr;
  
   testInfo("testReadMessage_illegalBuffer");
   checkResult("create socket", socket != SOCKET_ERROR);
   memset(&dstAddr, 0, sizeof(dstAddr));
   dstAddr.sin_addr.s_addr = inet_addr(SUCCESS_IP);
   checkResult("send echo", IcmpSendEcho(socket, (INT8 *)&dstAddr.sin_addr.S_un.S_addr, ICMP_ID, ICMP_SEQ, ICMP_DATA, ICMP_DATA_LEN)  != SOCKET_ERROR);
   checkResult("wait",  IcmpWait(socket, 5000) == 1);
   checkResult("read", IcmpReadMessage(socket, (IcmpResponseMessage *)NULL) == SOCKET_ERROR);
   checkResult("check result", IcmpGetLastError() == WSAEINVAL);
   checkResult("close socket", IcmpCloseSocket(socket) != SOCKET_ERROR);
   return;
} /* testReadMessage_illegalBuffer */


/*
 * Test IcmpDecodeResponseMessage function providing proper set of parameters.
 * The echo request is sent so after a while response is received. The result is than
 * decoded.
 */
static void testDecodeMessage_success(void)
{
   SOCKET socket = IcmpCreateSocket(); 
   IcmpResponseMessage response;
   struct sockaddr_in dstAddr;
   UINT8  srcAddress[4];
   UINT8  dstAddress[4];
   UINT8  type;
   UINT8  code;
   UINT16 identifier;
   UINT16 sequenceNumber;
   INT8   dataBuffer[ICMP_ECHO_MAX_DATA_LEN];
   size_t dataLength;
   
   testInfo("testDecodeMessage_success");
   checkResult("create socket", socket != SOCKET_ERROR);
   memset(&dstAddr, 0, sizeof(dstAddr));
   dstAddr.sin_addr.s_addr = inet_addr(SUCCESS_IP);
   checkResult("send echo", IcmpSendEcho(socket, (INT8 *)&dstAddr.sin_addr.S_un.S_addr, ICMP_ID, ICMP_SEQ, ICMP_DATA, ICMP_DATA_LEN)  != SOCKET_ERROR);
   checkResult("wait",  IcmpWait(socket, 5000) == 1);
   checkResult("read", IcmpReadMessage(socket, &response) != SOCKET_ERROR);
   checkResult("close socket", IcmpCloseSocket(socket) != SOCKET_ERROR);
   checkResult("decode message", IcmpDecodeResponseMessage(&response, srcAddress, dstAddress, &type, &code, &identifier, &sequenceNumber, dataBuffer, ICMP_ECHO_MAX_DATA_LEN, &dataLength) == TRUE);
   checkResult("message id", identifier == ICMP_ID);
   checkResult("message seq", sequenceNumber == ICMP_SEQ);
   checkResult("message type", type == ICMP_ECHOREPLY);
   checkResult("message code", code == 0);
   checkResult("message data length", dataLength == ICMP_DATA_LEN);
   checkResult("message type", memcmp(dstAddress, &dstAddr.sin_addr.S_un.S_addr, sizeof(dstAddress)) == 0);
   if (dataLength == ICMP_DATA_LEN) 
      checkResult("message data", strncmp(dataBuffer, ICMP_DATA, dataLength) == 0);
   return;
} /* testDecodeMessage_success */


/*
 * Test IcmpDecodeResponseMessage function providing proper set of parameters.
 * The echo request is sent so after a while response is received. The result is than
 * decoded.
 */
static void testDecodeMessage_unreachable(void)
{
   SOCKET socket = IcmpCreateSocket(); 
   IcmpResponseMessage response;
   struct sockaddr_in dstAddr;
   UINT8 srcAddress[4];
   UINT8 dstAddress[4];
   UINT8 type;
   UINT8 code;
   UINT16 identifier;
   UINT16 sequenceNumber;
   INT8   dataBuffer[ICMP_ECHO_MAX_DATA_LEN];
   size_t dataLength;
   
   testInfo("testDecodeMessage_unreachable");
   checkResult("create socket", socket != SOCKET_ERROR);
   memset(&dstAddr, 0, sizeof(dstAddr));
   dstAddr.sin_addr.s_addr = inet_addr(UNRECHABLE_IP);
   checkResult("send echo", IcmpSendEcho(socket, (INT8 *)&dstAddr.sin_addr.S_un.S_addr, ICMP_ID, ICMP_SEQ, ICMP_DATA, ICMP_DATA_LEN)  != SOCKET_ERROR);
   checkResult("wait",  IcmpWait(socket, 5000) == 1);
   checkResult("read", IcmpReadMessage(socket, &response) != SOCKET_ERROR);
   checkResult("close socket", IcmpCloseSocket(socket) != SOCKET_ERROR);
   checkResult("decode message", IcmpDecodeResponseMessage(&response, srcAddress, dstAddress, &type, &code, &identifier, &sequenceNumber, dataBuffer, ICMP_ECHO_MAX_DATA_LEN, &dataLength) == TRUE);
   checkResult("message id", identifier == ICMP_ID);
   checkResult("message seq", sequenceNumber == ICMP_SEQ);
   checkResult("message type", type == ICMP_DESTINATIONUNREACHABLE);
   checkResult("message type", memcmp(dstAddress, &dstAddr.sin_addr.S_un.S_addr, sizeof(dstAddress)) == 0);
   return;
} /* testDecodeMessage_unreachable */


int main()
{
   int result;
   
   result = IcmpInitialize();
   if (result == 0)
   {
      testsInit();
      testMutex_simple();
      testMutex_double();
      testGetErrrorString();
      testCreateCloseSocket_success();
      testGetSocketError_wrongSocket();
      testGetSocketError_success();
      testSendEcho_success();
      testSendEcho_wrongSocket();
      testSendEcho_illegalData();
      testSendEcho_illegalAddress();
      testSendEcho_noAnswer();
      testWaitForResponse_success();
      testWaitForResponse_timeout();
      testWaitForResponse_illegalSocket();
      testReadMessage_success();
      testReadMessage_illegalBuffer();
      testDecodeMessage_success();
      testDecodeMessage_unreachable();
      testsExit();
      IcmpRelease();
   }
   else  printf("WSAStartup failed with error: %d\n", result);
   return 0;
}

