/*
 * TimeCheckTest.java 
 *
 * Copyright (c) Verax Systems.
 * All rights reserved.
 *
 * This software is furnished under a license. Use, duplication,
 * disclosure and all other uses are restricted to the rights
 * specified in the written license agreement.
 */
package com.veraxsystems.icmp.ping;

import com.veraxsystems.icmp.jni.IcmpEchoResponse;
import com.veraxsystems.icmp.ping.Ping;
import com.veraxsystems.icmp.ping.PingResponseHandler;
import com.veraxsystems.icmp.ping.exception.PingErrorException;
import com.veraxsystems.icmp.ping.handlers.PingRequest;

import java.lang.reflect.Array;
import java.net.Inet4Address;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Arrays;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.Semaphore;
import java.util.concurrent.TimeUnit;

/**
 * Unit test Ping class
 */
public class TimeCheckTest
{
   
   private static final String testData = "PingTest: DATA";
   private static final String testHosts[] = { 
       "127.0.0.1",
       "8.8.8.8",
       "1.2.3.4",
       "192.168.9.1",
       "news.yahoo.com",
       "www.google.com",
       "8.8.8.8",
       "www.vertel.com",
       "www.cnn.com",
       "www.foxnews.com",
       "www.accuweather.com",
       "www.huffingtonpost.com",
       "en.wikipedia.org",
       "www.hostitwise.com",
       "www.advancedgaragedoorinc.com",
       "www.bidobee.com",
       "www.imedicileather.com",
       "www.rushrash.com",
       "news.bbc.co.uk",
       "www.usatoday.com",
       "www.msnbc.com",
       "www.ikea.com",
       "www.veraxsystems.com",
       "10.0.0.0"
   };
   static private InetAddress[] destinationAddresses = null;
   static private int times[] = null;
   static private int timesMax = 0;
   static private int pos = 0;
   static private int count = 1000;
   static private Semaphore sem = null;
   private static int loopsCount = 20;
   
   
   static class ResponseHandler implements PingResponseHandler
   {
      CountDownLatch counter = null; 
      int responsesNumber;

      public ResponseHandler(CountDownLatch counterVal) 
      {
         counter = counterVal;
         responsesNumber = 0;
         return;
      }
      
      public void updateTimes(int responseTime)
      {
         if (responseTime < times.length)
         {
            times[responseTime] += 1;
            if (timesMax < responseTime) timesMax = responseTime;
         }
         if (responseTime == 0)
         {
            int xxxx  = 1; // Here breakpoint may be set
         }
      }
      
      public void handleResponse(PingRequest request, IcmpEchoResponse response) 
      {
         //System.out.println("Got response for address " + request.getAddress().getHostAddress() + " from " + response.getSourceAddress().getHostAddress() + " Status: " + response.getStatus() + " id: " + request.getMessageId() + " time:" + (TimeUtil.currentTimeMillis() - request.getSendTime()));
         updateTimes((int)(response.getResponseTime() - request.getSendTime()));
         /*
         if ((response.getResponseTime() - request.getSendTime()) == 0)
            System.out.println("Zero time : " + (response.creationTimeNano - request.sendTimeNano) + " " +  request.getAddress().getHostAddress());
          */
         counter.countDown();
         responsesNumber += 1;
         sem.release();
      }
      
      public void handleTimeout(PingRequest request) {
         //System.out.println("Got timeout for address " + request.getAddress().getHostAddress()+ " id: " + request.getMessageId());
         counter.countDown();
         responsesNumber += 1;
         sem.release();
      }
      
      public void handleException(PingRequest request, Exception exception) {
         //System.out.println("Got exception for address " + request.getAddress().getHostAddress() + " id: " + request.getMessageId() + " " + exception.toString());
         counter.countDown();
         responsesNumber += 1;
         sem.release();
      }

      public int getResponsesNumber() {
         return responsesNumber;
      }
      
   }
   
   
   /**
    * Initialize long run test
    *
    * @param testName name of the test case
    */
   public static void init()
   {
      destinationAddresses = new InetAddress[testHosts.length];
      times = new int[5000];
      
      Arrays.fill(times, 0);
      for(int i = 0; i < testHosts.length; i++)
      {
          try {
              destinationAddresses[i] = Inet4Address.getByName(testHosts[i]);
          } catch (UnknownHostException ex) {
              System.out.println("Unknown host: " + testHosts[i] + " - ignored");
          }
      };
      sem = new Semaphore(destinationAddresses.length);
      return;
   }
   
   private static synchronized int getPos()
   {
      int result = pos;
      pos = (pos + 1) % destinationAddresses.length;
      return result;
   }
   
   
   /**
    * Test Ping echo requests using predefined set of hosts.
    */
   public static void main(String[] args) 
   {
       init();
       for(int i = 0; i < loopsCount; i++)
       {
           try {
               CountDownLatch counter = new CountDownLatch(count);
               ResponseHandler handler = new ResponseHandler(counter);
               
               Ping.init();
               System.out.println("Ping loop " + i);
               try 
               {
                   for(int j = 0; j < count; j++)
                   {
                       sem.acquire();
                       Ping.echo(destinationAddresses[getPos()], "TestData".getBytes(), 1000, handler);
                   };
                   // Success when counter reaches 0 before timeout
                   counter.await(1100, TimeUnit.MILLISECONDS);
               }
               catch (InterruptedException ex) 
               {
                   System.out.println("Test interrupted");
               }
               finally
               {
                   Ping.release();
                   counter = null;
                   handler = null;
               }
           }
           catch (PingErrorException ex) 
           {
               ex.printStackTrace();
           }
       };
       for(int j = 0; j < timesMax; j++)
          System.out.println("Time[" + j + "]=" + times[j]);
   }   
}
